@@ -1,42 +1,41 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
- *
- * This file is part of libxls -- A multiplatform, C/C++ library
- * for parsing Excel(TM) files.
- *
- * Redistribution and use in source and binary forms, with or without modification, are
- * permitted provided that the following conditions are met:
- *
- *    1. Redistributions of source code must retain the above copyright notice, this list of
- *       conditions and the following disclaimer.
- *
- *    2. Redistributions in binary form must reproduce the above copyright notice, this list
- *       of conditions and the following disclaimer in the documentation and/or other materials
- *       provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY David Hoerl ''AS IS'' AND ANY EXPRESS OR IMPLIED
- * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL David Hoerl OR
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
- * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
- * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
- * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
- * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  * Copyright 2004 Komarov Valery
  * Copyright 2006 Christophe Leitienne
+ * Copyright 2008-2017 David Hoerl
  * Copyright 2013 Bob Colbert
- * Copyright 2008-2013 David Hoerl
+ * Copyright 2013-2018 Evan Miller
+ *
+ * This file is part of libxls -- A multiplatform, C/C++ library for parsing
+ * Excel(TM) files.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *
+ *    1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ *
+ *    2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ''AS
+ * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+ * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+ * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
+ * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
+ * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
+ * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
+ * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  */
 
 #ifndef XLS_STRUCT_INC
 #define XLS_STRUCT_INC
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 #include "libxls/ole.h"
 
 #define XLS_RECORD_EOF          0x000A
@@ -54,6 +53,7 @@ extern "C" {
 #define XLS_RECORD_PALETTE      0x0092
 #define XLS_RECORD_MULRK        0x00BD
 #define XLS_RECORD_MULBLANK     0x00BE
+#define XLS_RECORD_RSTRING      0x00D6
 #define XLS_RECORD_DBCELL       0x00D7
 #define XLS_RECORD_XF           0x00E0
 #define XLS_RECORD_MSODRAWINGGROUP   0x00EB
@@ -127,15 +127,15 @@ typedef struct BOUNDSHEET
     DWORD	filepos;
     BYTE	type;
     BYTE	visible;
-    char	name[];
+    char	name[1];
 }
 BOUNDSHEET;
 
 typedef struct ROW
 {
     WORD	index;
-    WORD	fcell;
-    WORD	lcell;
+    WORD	fcell; // first cell, 0-indexed
+    WORD	lcell; // last cell, 1-indexed
     WORD	height;
     WORD	notused;
     WORD	notused2; //used only for BIFF3-4
@@ -187,7 +187,7 @@ typedef struct RK
     WORD	row;
     WORD	col;
     WORD	xf;
-    DWORD_UA value;
+    DWORD   value;
 }
 RK;
 
@@ -197,8 +197,8 @@ typedef struct MULRK
     WORD	col;
 	struct {
 		WORD	xf;
-		DWORD_UA value;
-	}		rk[];
+		DWORD   value;
+	}		rk[1];
 	//WORD	last_col;
 }
 MULRK;
@@ -207,7 +207,7 @@ typedef struct MULBLANK
 {
     WORD	row;
     WORD	col;
-    WORD	xf[];
+    WORD	xf[1];
 	//WORD	last_col;
 }
 MULBLANK;
@@ -225,10 +225,9 @@ typedef struct LABEL
     WORD	row;
     WORD	col;
     WORD	xf;
-    BYTE	value[]; // var
+    BYTE	value[1]; // var
 }
 LABEL;
-typedef LABEL LABELSST;
 
 typedef struct BOOLERR
 {
@@ -244,7 +243,7 @@ typedef struct SST
 {
     DWORD	num;
     DWORD	numofstr;
-    BYTE	strings[];
+    BYTE	strings[1];
 }
 SST;
 
@@ -292,7 +291,12 @@ typedef struct COLINFO
     WORD	width;
     WORD	xf;
     WORD	flags;
-    WORD	notused;
+/* There should be an unused WORD field at the end here. However, some files in
+ * the wild report it as a BYTE, which results in a boundary-check parse error.
+ * Since the value is ignored anyway, we'll just pretend it was never there.
+ *
+ * See issue https://github.com/evanmiller/libxls/issues/27
+ */
 }
 COLINFO;
 
@@ -316,14 +320,14 @@ typedef struct FONT
     BYTE	family;
     BYTE	charset;
     BYTE	notused;
-    char    name[];
+    char    name[1];
 }
 FONT;
 
 typedef struct FORMAT
 {
     WORD	index;
-    char	value[];
+    char	value[1];
 }
 FORMAT;
 
@@ -496,6 +500,10 @@ typedef struct xlsWorkBook
 
 	char		*summary;		// ole file
 	char		*docSummary;	// ole file
+
+    void        *converter;
+    void        *utf16_converter;
+    void        *utf8_locale;
 }
 xlsWorkBook;
 
@@ -534,8 +542,4 @@ xlsSummaryInfo;
 
 typedef void (*xls_formula_handler)(WORD bof, WORD len, BYTE *formula);
 
-#ifdef __cplusplus
-} // extern c block
-#endif
-
 #endif
